﻿using System.Collections.Generic;

namespace HeuristicLab.PluginInfrastructure {
  /// <summary>
  /// Interface for a plugin load mechanism.
  /// </summary>
  public interface IPluginLoader {
    /// <summary>
    /// Returns all plugins found.
    /// </summary>
    IEnumerable<IPlugin> Plugins { get; }

    /// <summary>
    /// Returns all Applications found.
    /// </summary>
    IEnumerable<IApplication> Applications { get; }

    /// <summary>
    /// Load plugins, definied by the given AssemblyInfos.
    /// </summary>
    /// <param name="assemblyInfos">AssemblyInfos for the assemblies, which should be loaded.</param>
    void LoadPlugins(IEnumerable<AssemblyInfo> assemblyInfos);

    /// <summary>
    /// Searches for .dll/.exe files and validates them.
    /// </summary>
    /// <param name="basePath">Path of the root directory for the assemblies to validate.</param>
    /// <returns>A IEnumerable of AssemblyInfos, which contains all loadable assemblies. </returns>
    IEnumerable<AssemblyInfo> Validate(string basePath);
  }
}
