﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

namespace HeuristicLab.PluginInfrastructure {
  [Serializable]
  public class ApplicationRunner : Runner {
    /// <summary>
    /// Arguments for the StartApplication.
    /// </summary>
    public ICommandLineArgument[] Args { get; set; }

    /// <summary>
    /// The application which should run in child process.
    /// </summary>
    public IApplication StartApplication {
      get {
        using (var memStream = new MemoryStream(serializedStartApplication)) {
          return (IApplication)formatter.Deserialize(memStream);
        }
      }
      set {
        using (var ms = new MemoryStream()) {
          formatter.Serialize(ms, value);
          serializedStartApplication = ms.ToArray();
        }
      }
    }
    // Encapsulated application is necessary, because it is not possible to 
    // instantly deserialize the application, before all assemblies are loaded.
    private byte[] serializedStartApplication = new byte[0];

    protected override void Execute() {
      StartApplication.Run(Args);
    }

    protected override void OnRunnerJob(RunnerJob runnerJob) {
      switch (runnerJob) {
        case RunnerJob.Cancel: StartApplication.OnCancel(); break;
        case RunnerJob.Pause:  StartApplication.OnPause (); break;
        case RunnerJob.Resume: StartApplication.OnResume(); break;
      }
    }
  }
}
