﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Threading;

namespace HeuristicLab.PluginInfrastructure {
  [Serializable]
  public abstract class Runner : IRunner {

    [NonSerialized]
    private Thread listener;

    [NonSerialized]
    private Thread executor;

    [NonSerialized]
    protected static IFormatter formatter = new BinaryFormatter();
    
    public bool QuietMode { get; set; }
     
    public IEnumerable<AssemblyInfo> AssembliesToLoad { get; set; }

    public static void Serialize(IRunner runner, Stream stream) => formatter.Serialize(stream, runner);
    public static IRunner Deserialize(Stream stream = null) => (IRunner)formatter.Deserialize(stream);
    public static void Pause(Stream stream) => formatter.Serialize(stream, RunnerJob.Pause);
    public static void Resume(Stream stream) => formatter.Serialize(stream, RunnerJob.Resume);
    public static void Cancel(Stream stream) => formatter.Serialize(stream, RunnerJob.Cancel);

    public void Run() {
      IPluginLoader loader = PluginLoaderFactory.Create();
      loader.LoadPlugins(AssembliesToLoad);
      StartExecutor();
      StartListener();
      executor.Join();
    }

    protected abstract void Execute();
    protected abstract void OnRunnerJob(RunnerJob runnerJob);

    private void StartExecutor() {
      executor = new Thread(Execute);
      executor.IsBackground = false;
      executor.Start();
    }
    private void StartListener() {
      listener = new Thread(() => {
        Stream stdin = Console.OpenStandardInput();
        while (executor.IsAlive) 
          OnRunnerJob((RunnerJob)formatter.Deserialize(stdin));
      });
      listener.IsBackground = true;
      listener.Start();
    }

  }
}
